@@ -9,7 +9,9 @@
 ## Licence:     This program is free software; you can redistribute it and/or
 ##              modify it under the same terms as Perl itself
 ##
-## This module is actualy XML::Parser::Lite. It's just here for convenience.
+## This module is actualy XML::Parser::Lite (with some updates). It's here
+## just for convenience.
+##
 ## See original code at CPAN for full source and POD.
 ##
 ## This module will be used when XML::Parser is not installed.
@@ -23,37 +25,51 @@
 #
 # $Id: Lite.pm,v 1.4 2001/10/15 21:25:05 paulk Exp $
 #
+# Changes: Graciliano M. P. <gm@virtuasites.com.br>
+#
 # ======================================================================
 
 package XML::Smart::Parser ;
 
+no warnings ;
 use strict;
 use vars qw($VERSION);
-$VERSION = sprintf("%d.%s", map {s/_//g; $_} q$Name: release-0_55-public $ =~ /-(\d+)_([\d_]+)/);
+$VERSION = 1.2 ;
+
+my(@parsed , @stack, $level) ;
+
+ &compile();
 
 sub new { 
-  my $self = shift;
-  my $class = ref($self) || $self;
-  return $self if ref $self;
+  my $this = shift;
+  my $class = ref($this) || $this;
+  return $this if ref $this;
+  $this = bless {} => $class ;
 
-  $self = bless {} => $class;
-  my %parameters = @_;
-  $self->setHandlers(); # clear first 
-  $self->setHandlers(%{$parameters{Handlers} || {}});
-  return $self;
+  my %args = @_ ;
+  $this->setHandlers(%args) ;
+  
+  $this->{NOENTITY} = 1 ;
+
+  return $this ;
 }
 
 sub setHandlers {
-  my $self = shift; 
-  no strict 'refs'; local $^W;
-  unless (@_) { foreach (qw(Start End Char Final Init)) { *$_ = sub {} } }
-  while (@_) { my($name => $func) = splice(@_, 0, 2); *$name = defined $func ? $func : sub {} }
-  return $self;
+  my $this = shift ;
+  my %args = @_;
+    
+  $this->{Init}  = $args{Init} || sub{} ;
+  $this->{Start} = $args{Start} || sub{} ;
+  $this->{Char}  = $args{Char} || sub{} ;
+  $this->{End}   = $args{End} || sub{} ;
+  $this->{Final} = $args{Final} || sub{} ;
+  
+  return 1 ;
 }
 
 sub regexp {
-  my $patch = shift || '';
-  my $package = __PACKAGE__;
+  my $patch = shift || '' ;
+  my $package = __PACKAGE__ ;
 
   my $TextSE = "[^<]+";
   my $UntilHyphen = "[^-]*-";
@@ -73,12 +89,12 @@ sub regexp {
   my $PI_Tail = "\\?>|$S1$UntilQMs(?:[^>?]$UntilQMs)*>";
   my $DT_ItemSE = "<(?:!(?:--$Until2Hyphens>|[^-]$MarkupDeclCE)|\\?$Name(?:$PI_Tail))|%$Name;|$S";
   my $DocTypeCE = "$DT_IdentSE(?:$S)?(?:\\[(?:$DT_ItemSE)*](?:$S)?)?>?";
-  my $DeclCE = "--(?:$CommentCE)?|\\[CDATA\\[(?:$CDATA_CE)?|DOCTYPE(?:$DocTypeCE)?";
+  my $DeclCE = "--(?:$CommentCE)?|\\[CDATA\\[(?:($CDATA_CE)(?{${package}::char_CDATA(\$2)}))?|DOCTYPE(?:$DocTypeCE)?";
   my $PI_CE = "$Name(?:$PI_Tail)?";
 
-  my $EndTagCE = "($Name)(?{${package}::end(\$2)})(?:$S)?>";
+  my $EndTagCE = "($Name)(?{${package}::end(\$3)})(?:$S)?>";
   my $AttValSE = "\"([^<\"]*)\"|'([^<']*)'";
-  my $ElemTagCE = "($Name)(?:$S($Name)(?:$S)?=(?:$S)?(?:$AttValSE)(?{[\@{\$^R||[]},\$4=>defined\$5?\$5:\$6]}))*(?:$S)?(/)?>(?{${package}::start(\$3,\@{\$^R||[]})})(?{\${7} and ${package}::end(\$3)})";
+  my $ElemTagCE = "($Name)(?:$S($Name)(?:$S)?=(?:$S)?(?:$AttValSE)(?{[\@{\$^R||[]},\$5=>defined\$6?\$6:\$7]}))*(?:$S)?(/)?>(?{${package}::start(\$4,\@{\$^R||[]})})(?{\${8} and ${package}::end(\$4)})";
   my $MarkupSPE = "<(?:!(?:$DeclCE)?|\\?(?:$PI_CE)?|/(?:$EndTagCE)?|(?:$ElemTagCE)?)";
 
   "(?:($TextSE)(?{${package}::char(\$1)}))$patch|$MarkupSPE";
@@ -93,36 +109,51 @@ sub compile { local $^W;
   *compile = sub {};
 }
 
-setHandlers();
-compile();
-
 sub parse { 
-  init(); 
-  parse_re($_[1]);
-  final(); 
+  my $this = shift ;
+  
+  @parsed = () ;
+  
+  init();
+  parse_re($_[0]);
+  final();
+
+  no strict qw(refs);
+  
+  my $final = pop(@parsed) ; pop(@parsed) ;
+
+  for (my $i = 0 ; $i <= $#parsed ; $i+=2) {
+    my $args = $parsed[$i+1] ;
+    &{$this->{$parsed[$i]}}($this , (ref($args) ? @{$args} : $args) ) ;
+  }
+
+  @parsed = () ;
+
+  return &{$this->{Final}}($this, @{$final}) ;
 }
 
-my(@stack, $level);
-
-sub init { 
+sub init {
   @stack = (); $level = 0;
-  Init(__PACKAGE__, @_);  
+  push(@parsed , 'Init' , [@_]) ;
+  return ;
 }
 
-sub final { 
+sub final {
   die "not properly closed tag '$stack[-1]'\n" if @stack;
   die "no element found\n" unless $level;
-  Final(__PACKAGE__, @_) 
+  push(@parsed , 'Final' , [@_]) ;
+  return ;
 } 
 
-sub start { 
+sub start {
   die "multiple roots, wrong element '$_[0]'\n" if $level++ && !@stack;
   push(@stack, $_[0]);
-  Start(__PACKAGE__, @_); 
+  push(@parsed , 'Start' , [@_]) ;
+  return ;
 }
 
-sub char { 
-  Char(__PACKAGE__, $_[0]), return if @stack;
+sub char {
+  push(@parsed , 'Char' , [@_]) , return if @stack;
 
   for (my $i=0; $i < length $_[0]; $i++) {
     die "junk '$_[0]' @{[$level ? 'after' : 'before']} XML element\n"
@@ -130,12 +161,18 @@ sub char {
   }
 }
 
-sub end { 
+sub char_CDATA {
+  &char( substr($_[0] , 0 , -3) ) ;
+}
+
+sub end {
   pop(@stack) eq $_[0] or die "mismatched tag '$_[0]'\n";
-  End(__PACKAGE__, $_[0]);
+  push(@parsed , 'End' , [@_]) ;
+  return ;
 }
 
 # ======================================================================
 
 1;
 
+
